@@ -1,24 +1,20 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
 """Module for declaring the datatype."""
 import base64
-from typing import (
-    Any,
-    Union,
-)
+from typing import Any
 
 import numpy as np
-import pythonflow as pf
 from scipy.sparse import coo_matrix
 
 from qctrlcommons.graph import Graph
@@ -179,6 +175,41 @@ class NumpyArray(DataType):
         ).reshape(obj["shape"])
 
 
+class QuantumObject(DataType):
+    """Represent QuTiP QuantumObject Model."""
+
+    def can_encode(self, obj: Any) -> bool:
+        """Checks that the object can be encoded with this class.
+        If it has attribute ``full`` it can be encoded.
+
+        Parameters
+        ----------
+        obj : Any
+            object to be examined.
+
+        Returns
+        -------
+        bool
+            True if the object can be encoded, False otherwise.
+
+        Raises
+        ------
+        RuntimeError
+            if the data type is `None`.
+        """
+
+        return hasattr(obj, "full")
+
+    def encode(self, obj):
+        return obj.full()
+
+    def can_decode(self, obj) -> bool:
+        return False
+
+    def decode(self, obj):
+        raise NotImplementedError
+
+
 class NumpyComplexNumber(DataType):
     """Represent NumpyComplexNumber Model."""
 
@@ -245,9 +276,13 @@ class ComplexNumber(DataType):
 
 
 class GraphDataType(DataType):
-    """Represent GraphDataType for pythonflow Graph."""
+    """
+    Custom data type for `Graph` object
+    """
 
-    _type = pf.core.Graph
+    _type = Graph
+    # keep the `object_key` as 'pythonflow_graph` to backward support
+    # pythonflow graph serialization for older version (before 11.3.0) Q-CTRL python package
     object_key = "pythonflow_graph"
 
     def _parse_kwargs(self, operations, kwarg_value: dict):
@@ -272,7 +307,7 @@ class GraphDataType(DataType):
     def _rebuild_operations(self, operations: dict):
         """
         Checks operations to see if any special reference nodes are present.
-        If present it repalces them with the real node values.
+        If present it replaces them with the real node values.
         """
         for operation in operations.values():
             operation.input_kwargs.update(
@@ -281,13 +316,14 @@ class GraphDataType(DataType):
 
         return operations
 
-    def encode(self, obj: Union[pf.Graph, Graph]) -> dict:
-        """Convert pythonflow graph to dict, all the operations from graph will
-        be encoded with OpeationDataType.
+    def encode(self, obj: Graph) -> dict:
+        """
+        Convert graph to dict, all the operations from graph will
+        be encoded with OperationDataType.
 
         Parameters
         ----------
-        obj: Union[pf.Graph, Graph]
+        obj: Graph
             object to be encoded.
 
         Returns
@@ -302,7 +338,8 @@ class GraphDataType(DataType):
         }
 
     def decode(self, obj: dict) -> Graph:
-        """Decode to pythonflow graph.
+        """
+        Decode to Graph.
 
         Parameters
         ----------
@@ -312,7 +349,7 @@ class GraphDataType(DataType):
         Returns
         -------
         Graph
-            a custom structure for encoding and decoding pythonflow graph.
+            Graph object.
 
         Raises
         ------
@@ -323,24 +360,26 @@ class GraphDataType(DataType):
             raise KeyError("Missing operations. It cannot be encoded to graph")
 
         operations = self._rebuild_operations(obj["operations"])
-        return Graph(operations, obj["dependencies"])
+        return Graph(operations=operations, dependencies=obj["dependencies"])
 
     def can_encode(self, obj) -> bool:
-        return isinstance(obj, (pf.core.Graph, Graph))
+        return isinstance(obj, Graph)
 
 
 class OperationDataType(DataType):
     """
-    Wrapper class for pythonflow operation.
+    Wrapper class for operation.
     """
 
     _type = Operation
+    # keep the `object_key` as 'pythonflow_op` to backward support
+    # pythonflow operation serialization for older version (before 11.3.0) Q-CTRL python package
     object_key = "pythonflow_op"
 
     def _set_kwargs_reference(self, value: Any):
         """
-        Checks which kwargs contain values that represent NodeData and stores only the references
-        for those values.
+        Checks which kwargs contain values that represent
+        NodeData and stores only the references for those values.
         """
         if isinstance(value, NodeData):
             return {"_kwarg_type": "node", "value": value.operation.name}
