@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -19,8 +19,8 @@ from typing import (
 import forge
 
 from qctrlcommons.exceptions import QctrlArgumentsValueError
-from qctrlcommons.node import node_data
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.node_data import Pwc
 from qctrlcommons.node.wrapper import NamedNodeData
 
 
@@ -30,8 +30,8 @@ class GetAttributeNode(Node):
 
     Only certain combinations of `value` and `attr` are supported:
 
-    `TensorPwc.values`
-        The tensor of values of a `TensorPwc`.
+    `Pwc.values`
+        The tensor of values of a `Pwc`.
 
     Parameters
     ----------
@@ -60,7 +60,7 @@ class GetAttributeNode(Node):
         value = kwargs.get("value")
         attr = kwargs.get("attr")
 
-        valid_combinations = [(node_data.TensorPwcNodeData, "values")]
+        valid_combinations = [(Pwc, "values")]
         for valid_value_type, valid_attr in valid_combinations:
             if isinstance(value, valid_value_type) and attr == valid_attr:
                 return NamedNodeData(_operation)
