@@ -1,28 +1,27 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
 """Module for Node."""
+import threading
 from typing import (
     Any,
-    Dict,
     Optional,
 )
 
 import forge
 
-from qctrlcommons.node.wrapper import (
-    NodeData,
-    Operation,
-)
+from qctrlcommons.node.wrapper import Operation
+
+LOCAL_THREADING = threading.local()
 
 
 class Node:
@@ -71,14 +70,20 @@ class Node:
         Can optionally perform validation of the inputs (which themselves will be `NodeData`
         objects, if they come from other graph function calls).
         """
-        return NodeData(_operation)
+        # create_node_data should be overridden by the subclass.
+        raise NotImplementedError
 
     @classmethod
     def create_pf(cls):
-        """Creates the corresponding pythonflow function."""
+        """Creates the corresponding operation function."""
 
         def func(name=None, **kwargs):
-            operation = Operation(cls.name, name=name, **kwargs)
+            operation = Operation(
+                graph=LOCAL_THREADING.default_graph,
+                operation_name=cls.name,
+                name=name,
+                **kwargs,
+            )
             return cls.create_node_data(_operation=operation, name=name, **kwargs)
 
         func.__doc__ = cls.__doc__
@@ -89,55 +94,23 @@ class Node:
 
         return func
 
-    def _evaluate_inputs(self, execution_context: "ExecutionContext") -> Dict:
-        """Evaluates the args and kwargs. Called before evaluating the node
-        itself.
-
-        Parameters
-        ----------
-        execution_context : ExecutionContext
-            helper class for evaluating the node value.
-
-        Returns
-        -------
-        Dict
-            the kwargs
+    @classmethod
+    def create_graph_method(cls):
+        """
+        Creates a callable that can be attached as a method to the Graph class for creating nodes of
+        this type.
         """
 
-        def _recursive_evaluate(arg):
-            if isinstance(arg, Node):
-                return arg.evaluate(execution_context)
-            if isinstance(arg, list):
-                return [_recursive_evaluate(element) for element in arg]
-            if isinstance(arg, dict):
-                return {key: _recursive_evaluate(arg[key]) for key in arg}
-            return arg
+        def func(self, name=None, **kwargs):
+            operation = Operation(
+                graph=self, operation_name=cls.name, name=name, **kwargs
+            )
+            return cls.create_node_data(_operation=operation, name=name, **kwargs)
 
-        kwargs = _recursive_evaluate(self.input_kwargs)
+        func.__doc__ = cls.__doc__
+        func.__name__ = cls.name  # pylint:disable=non-str-assignment-to-dunder-name
+        sig = forge.sign(forge.arg("self"), *cls.args, **cls.kwargs)
+        func = sig(func)
+        func = forge.returns(cls.rtype)(func)
 
-        return kwargs
-
-    def evaluate(self, execution_context: "ExecutionContext") -> Any:
-        """Get the value from the Node itself.
-
-        Parameters
-        ----------
-        execution_context : ExecutionContext
-            helper class for evaluating the value of node.
-
-        Returns
-        -------
-        Any
-            the value of the node.
-        """
-        # check if it is evaluated
-        if execution_context.is_recorded(self):
-            return execution_context.get_recorded_value(self)
-
-        kwargs = self._evaluate_inputs(execution_context)
-
-        node_function = getattr(execution_context.nodes, self.name, None)
-        value = node_function(**kwargs)
-
-        execution_context.record(self, value)
-        return value
+        return func
