@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -20,11 +20,10 @@ from typing import (
 import forge
 import numpy as np
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
 from qctrlcommons.node.node_data import (
-    TargetNodeData,
-    TensorNodeData,
+    Target,
+    Tensor,
 )
 from qctrlcommons.node.utils import validate_shape
 from qctrlcommons.preconditions import (
@@ -38,7 +37,7 @@ from qctrlcommons.preconditions import (
 )
 
 
-class TargetOperator(Node):
+class TargetOperation(Node):
     r"""
     Creates information about the target for system time evolution.
 
@@ -55,8 +54,6 @@ class TargetOperator(Node):
         The orthogonal projection matrix :math:`P` onto the subspace used for
         filter function calculations. If you provide a value then it must be
         Hermitian and idempotent. Defaults to the identity matrix.
-    name : str, optional
-        The name of the node.
 
     Returns
     -------
@@ -90,15 +87,16 @@ class TargetOperator(Node):
 
     name = "target"
     args = [
-        forge.arg("operator", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("operator", type=Union[np.ndarray, Tensor]),
         forge.arg("filter_function_projector", type=Optional[np.ndarray], default=None),
     ]
-    rtype = types.Target
+    kwargs = {}  # Target should not accept `name` as parameter
+    rtype = Target
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         operator = kwargs.get("operator")
-        values_shape = validate_shape(operator, "operator")
+        value_shape = validate_shape(operator, "operator")
 
         if isinstance(operator, np.ndarray):
             check_argument_numeric(operator, "operator")
@@ -107,10 +105,10 @@ class TargetOperator(Node):
         else:
             check_operator(operator, "operator")
             check_argument(
-                len(values_shape) == 2,
+                len(value_shape) == 2,
                 "The operator must be a matrix, not a batch.",
                 {"operator": operator},
-                extras={"operator.shape": values_shape},
+                extras={"operator.shape": value_shape},
             )
 
         filter_function_projector = kwargs.get("filter_function_projector")
@@ -122,7 +120,7 @@ class TargetOperator(Node):
                 filter_function_projector, "filter_function_projector"
             )
 
-        return TargetNodeData(_operation, values_shape=values_shape)
+        return Target(_operation, value_shape=value_shape)
 
 
 class AnchoredDifferenceBoundedVariables(Node):
@@ -162,9 +160,9 @@ class AnchoredDifferenceBoundedVariables(Node):
 
     See Also
     --------
+    bounded_optimization_variable : Create bounded optimization variables.
     :func:`~qctrl.dynamic.namespaces.FunctionNamespace.calculate_optimization` : Find the minimum
         of generic functions.
-    bounded_optimization_variable : Create bounded optimization variables.
     unbounded_optimization_variable : Create unbounded optimization variables.
     """
 
@@ -176,12 +174,12 @@ class AnchoredDifferenceBoundedVariables(Node):
         forge.arg("upper_bound", type=float),
         forge.arg("difference_bound", type=float),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         count = kwargs.get("count")
-        return TensorNodeData(_operation, shape=(count,))
+        return Tensor(_operation, shape=(count,))
 
 
 class HessianMatrix(Node):
@@ -240,17 +238,17 @@ class HessianMatrix(Node):
     """
     name = "hessian_matrix"
     args = [
-        forge.arg("tensor", type=types.Tensor),
-        forge.arg("variables", type=List[types.Tensor]),
+        forge.arg("tensor", type=Tensor),
+        forge.arg("variables", type=List[Tensor]),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         tensor = kwargs.get("tensor")
         variables = kwargs.get("variables")
         check_argument(
-            isinstance(tensor, TensorNodeData),
+            isinstance(tensor, Tensor),
             "The tensor parameter must be a Tensor.",
             {"tensor": tensor},
         )
@@ -262,7 +260,7 @@ class HessianMatrix(Node):
         )
         check_argument_iteratable(variables, "variables")
         check_argument(
-            all(isinstance(variable, TensorNodeData) for variable in variables),
+            all(isinstance(variable, Tensor) for variable in variables),
             "Each of the variables must be a Tensor.",
             {"variables": variables},
         )
@@ -273,7 +271,7 @@ class HessianMatrix(Node):
             ]
         )
         shape = (variable_count, variable_count)
-        return TensorNodeData(_operation, shape=shape)
+        return Tensor(_operation, shape=shape)
 
 
 class Concatenate(Node):
@@ -298,10 +296,10 @@ class Concatenate(Node):
 
     name = "concatenate"
     args = [
-        forge.arg("tensors", type=List[Union[np.ndarray, types.Tensor]]),
+        forge.arg("tensors", type=List[Union[np.ndarray, Tensor]]),
         forge.arg("axis", type=int),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -329,4 +327,4 @@ class Concatenate(Node):
             + (sum([tensor_shape[axis] for tensor_shape in tensor_shapes]),)
             + tensor_shapes[0][axis + 1 :]
         )
-        return TensorNodeData(_operation, shape=shape)
+        return Tensor(_operation, shape=shape)
