@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -14,26 +14,35 @@
 from typing import (
     List,
     Optional,
+    Union,
 )
 
 import forge
 import numpy as np
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.deprecation import deprecated_parameter
 from qctrlcommons.node.node_data import (
-    StfNodeData,
-    TargetNodeData,
-    TensorNodeData,
-    TensorPwcNodeData,
+    Pwc,
+    Stf,
+    Target,
+    Tensor,
+)
+from qctrlcommons.node.utils import (
+    is_broadcastable,
+    validate_batch_and_value_shapes,
+    validate_broadcasted_shape,
+    validate_shape,
 )
-from qctrlcommons.node.utils import validate_batch_and_values_shapes
 from qctrlcommons.preconditions import (
     check_argument,
     check_sample_times,
 )
 
 
+@deprecated_parameter(  # deprecated 2021/07/14
+    deprecated_name="target_operator", new_name="target"
+)
 class InfidelityPwc(Node):
     r"""
     Creates the total infidelity of the given piecewise-constant system.
@@ -51,18 +60,20 @@ class InfidelityPwc(Node):
 
     Parameters
     ----------
-    hamiltonian : TensorPwc
+    hamiltonian : Pwc
         The control Hamiltonian :math:`H_{\mathrm c}(t)`. You can provide
         either a single Hamiltonian or a batch of them.
-    target_operator : Target
+    target : Target
         The object describing the target gate :math:`U_\mathrm{target}` and
         (optionally) the filter function projector :math:`P`. If you
         provide a batch of Hamiltonians, the function uses the same target
         for all the elements in the batch.
-    noise_operators : List[TensorPwc], optional
-        The perturbative noise operators :math:`\{N_j(t)\}`. You can omit this
-        list if there are no noises. If you provide a batch of Hamiltonians
-        rather than a single Hamiltonian, don't provide noise operators.
+    noise_operators : List[np.ndarray or Tensor or Pwc], optional
+        The perturbative noise operators :math:`\{N_j(t)\}`. The operators
+        in the list can either be single operators or batches of them. If
+        any of the noise operators or the Hamiltonian are batches, the batch
+        shapes must all be broadcastable. You can omit this list if there are
+        no noises.
     name : str, optional
         The name of the node.
 
@@ -71,15 +82,12 @@ class InfidelityPwc(Node):
     Tensor
         The total infidelity (operational infidelity plus filter function
         values) of the given system, with respect to the given target gate.
-        If you provide a batch of Hamiltonians, the function returns a
-        batch of infidelities contaning one infidelity for each Hamiltonian
-        of the input batch.
+        If you provide a batch of Hamiltonians or noise operators, the
+        function returns a batch of infidelities containing one infidelity
+        for each Hamiltonian and list of noise operators in the input batches.
 
     Warnings
     --------
-    This function currently doesn't support noise operators when you
-    provide a batch of Hamiltonians (rather than a single Hamiltonian).
-
     The Hessian matrix cannot currently be calculated for a graph which includes
     an `infidelity_pwc` node if the `hamiltonian` has degenerate eigenvalues at
     any segment.
@@ -87,9 +95,9 @@ class InfidelityPwc(Node):
     See Also
     --------
     infidelity_stf : Corresponding operation for `Stf` Hamiltonians.
-    time_evolution_operators_pwc : Unitary time evolution operators for quantum systems with
-        `TensorPwc` Hamiltonians.
     target : Define the target operation of the time evolution.
+    time_evolution_operators_pwc : Unitary time evolution operators for quantum systems with
+        `Pwc` Hamiltonians.
 
     Notes
     -----
@@ -137,44 +145,65 @@ class InfidelityPwc(Node):
 
     name = "infidelity_pwc"
     args = [
-        forge.arg("hamiltonian", type=types.TensorPwc),
-        forge.arg("target_operator", type=types.Target),
+        forge.arg("hamiltonian", type=Pwc),
+        forge.arg("target", type=Target),
         forge.arg(
-            "noise_operators", type=Optional[List[types.TensorPwc]], default=None
+            "noise_operators",
+            type=Optional[List[Union[np.ndarray, Tensor, Pwc]]],
+            default=None,
         ),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         hamiltonian = kwargs.get("hamiltonian")
-        target_operator = kwargs.get("target_operator")
+        target = kwargs.get("target")
         noise_operators = kwargs.get("noise_operators")
         check_argument(
-            isinstance(hamiltonian, TensorPwcNodeData),
-            "The Hamiltonian must be a TensorPwc.",
+            isinstance(hamiltonian, Pwc),
+            "The Hamiltonian must be a Pwc.",
             {"hamiltonian": hamiltonian},
         )
         check_argument(
-            isinstance(target_operator, TargetNodeData),
-            "The target_operator must be a Target.",
-            {"target_operator": target_operator},
+            isinstance(target, Target),
+            "The target must be a Target.",
+            {"target": target},
         )
-        batch_shape, _ = validate_batch_and_values_shapes(hamiltonian, "hamiltonian")
-        if noise_operators is not None:
+        batch_shape, _ = validate_batch_and_value_shapes(hamiltonian, "hamiltonian")
+        for noise_operator in noise_operators or []:
             check_argument(
-                len(noise_operators) == 0 or batch_shape == (),
-                "If you provide a batch of Hamiltonians, you can't provide noise operators.",
-                {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
-            )
-            check_argument(
-                all(isinstance(noise, TensorPwcNodeData) for noise in noise_operators),
-                "Each of the noise_operators must be a TensorPwc.",
+                isinstance(noise_operator, (np.ndarray, Tensor, Pwc)),
+                "Each of the noise_operators must be an array, a Tensor, or a Pwc.",
                 {"noise_operators": noise_operators},
             )
-        return TensorNodeData(_operation, shape=batch_shape)
+            if isinstance(noise_operator, Pwc):
+                noise_operator_batch_shape, _ = validate_batch_and_value_shapes(
+                    noise_operator, "noise_operators"
+                )
+            else:
+                noise_operator_batch_shape = validate_shape(
+                    noise_operator, "noise_operators"
+                )[:-2]
+            check_argument(
+                is_broadcastable(batch_shape, noise_operator_batch_shape),
+                "The batch shapes of the Hamiltonian and noise_operators must be broadcastable.",
+                {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
+            )
+
+            # Update the overall batch shape to take into account this noise operator (note that
+            # the check can't fail, since we already ensured the shapes were broadcastable, so we
+            # can pass dummy names).
+            batch_shape = validate_broadcasted_shape(
+                batch_shape, noise_operator_batch_shape, "", ""
+            )
+
+        return Tensor(_operation, shape=batch_shape)
 
 
+@deprecated_parameter(  # deprecated 2021/07/14
+    deprecated_name="target_operator", new_name="target"
+)
 class InfidelityStf(Node):
     r"""
     Creates the total infidelity of a given system with a sampleable Hamiltonian.
@@ -190,28 +219,33 @@ class InfidelityStf(Node):
     hamiltonian : Stf
         The control Hamiltonian :math:`H_{\mathrm c}(t)`. You can provide
         either a single Hamiltonian or a batch of them.
-    target_operator : Target
+    target : Target
         The object describing the target gate :math:`U_\mathrm{target}` and
         (optionally) the filter function projector :math:`P`. If you
         provide a batch of Hamiltonians, the function uses the same target
         for all the elements in the batch.
-    noise_operators : List[Stf], optional
-        The perturbative noise operators :math:`\{N_j(t)\}`. You can omit this
-        list if there are no noises.
+    noise_operators : List[np.ndarray or Tensor or Stf], optional
+        The perturbative noise operators :math:`\{N_j(t)\}`. The operators
+        in the list can either be single operators or batches of them. If
+        any of the noise operators or the Hamiltonian are batches, the batch
+        shapes must all be broadcastable. You can omit this list if there are
+        no noises.
     name : str, optional
         The name of the node.
 
     Returns
     -------
     Tensor
-        The infidelity of the given system with respect to the given target at the last time in
-        `sample_times`.
-        If you provide a batch of Hamiltonians, the function returns a batch of infidelities
-        contaning one infidelity for each Hamiltonian of the input batch.
+        The total infidelity (operational infidelity plus filter function
+        values) of the given system, with respect to the given target gate,
+        at the last time in `sample_times`.
+        If you provide a batch of Hamiltonians or noise operators, the
+        function returns a batch of infidelities containing one infidelity
+        for each Hamiltonian and list of noise operators in the input batches.
 
     See Also
     --------
-    infidelity_pwc : Corresponding operation for `TensorPwc` Hamiltonians.
+    infidelity_pwc : Corresponding operation for `Pwc` Hamiltonians.
     target : Define the target operation of the time evolution.
     time_evolution_operators_stf : Unitary time evolution operators for quantum systems with
         `Stf` Hamiltonians.
@@ -220,17 +254,21 @@ class InfidelityStf(Node):
     name = "infidelity_stf"
     args = [
         forge.arg("sample_times", type=np.ndarray),
-        forge.arg("hamiltonian", type=types.Stf),
-        forge.arg("target_operator", type=types.Target),
-        forge.arg("noise_operators", type=Optional[List[types.Stf]], default=None),
+        forge.arg("hamiltonian", type=Stf),
+        forge.arg("target", type=Target),
+        forge.arg(
+            "noise_operators",
+            type=Optional[List[Union[np.ndarray, Tensor, Stf]]],
+            default=None,
+        ),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         sample_times = kwargs.get("sample_times")
         hamiltonian = kwargs.get("hamiltonian")
-        target_operator = kwargs.get("target_operator")
+        target = kwargs.get("target")
         noise_operators = kwargs.get("noise_operators")
         check_sample_times(sample_times, "sample_times")
         check_argument(
@@ -239,20 +277,41 @@ class InfidelityStf(Node):
             {"sample_times": sample_times},
         )
         check_argument(
-            isinstance(hamiltonian, StfNodeData),
+            isinstance(hamiltonian, Stf),
             "The Hamiltonian must be an Stf.",
             {"hamiltonian": hamiltonian},
         )
         check_argument(
-            isinstance(target_operator, TargetNodeData),
-            "The target_operator must be a Target.",
-            {"target_operator": target_operator},
+            isinstance(target, Target),
+            "The target must be a Target.",
+            {"target": target},
         )
-        if noise_operators is not None:
+        batch_shape, _ = validate_batch_and_value_shapes(hamiltonian, "hamiltonian")
+        for noise_operator in noise_operators or []:
             check_argument(
-                all(isinstance(noise, StfNodeData) for noise in noise_operators),
-                "Each of the noise_operators must be an Stf.",
+                isinstance(noise_operator, (np.ndarray, Tensor, Stf)),
+                "Each of the noise_operators must be an array, a Tensor, or an Stf.",
                 {"noise_operators": noise_operators},
             )
-        batch_shape, _ = validate_batch_and_values_shapes(hamiltonian, "hamiltonian")
-        return TensorNodeData(_operation, shape=batch_shape)
+            if isinstance(noise_operator, Stf):
+                noise_operator_batch_shape, _ = validate_batch_and_value_shapes(
+                    noise_operator, "noise_operators"
+                )
+            else:
+                noise_operator_batch_shape = validate_shape(
+                    noise_operator, "noise_operators"
+                )[:-2]
+            check_argument(
+                is_broadcastable(batch_shape, noise_operator_batch_shape),
+                "The batch shapes of the Hamiltonian and noise_operators must be broadcastable.",
+                {"hamiltonian": hamiltonian, "noise_operators": noise_operators},
+            )
+
+            # Update the overall batch shape to take into account this noise operator (note that
+            # the check can't fail, since we already ensured the shapes were broadcastable, so we
+            # can pass dummy names).
+            batch_shape = validate_broadcasted_shape(
+                batch_shape, noise_operator_batch_shape, "", ""
+            )
+
+        return Tensor(_operation, shape=batch_shape)
