@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -26,12 +26,11 @@ import forge
 import numpy as np
 from scipy.sparse import coo_matrix
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
 from qctrlcommons.node.node_data import (
-    SparsePwcNodeData,
-    TensorNodeData,
-    TensorPwcNodeData,
+    Pwc,
+    SparsePwc,
+    Tensor,
 )
 from qctrlcommons.node.utils import (
     check_argument,
@@ -111,17 +110,17 @@ class EstimatedKrylovSubspaceDimensionArnoldi(Node):
 
     name = "estimated_krylov_subspace_dimension_arnoldi"
     args = [
-        forge.arg("hamiltonian_sample", type=Union[np.array, coo_matrix, types.Tensor]),
+        forge.arg("hamiltonian_sample", type=Union[np.array, coo_matrix, Tensor]),
         forge.arg(
             "lindblad_terms",
-            type=List[Tuple[float, Union[np.array, coo_matrix, types.Tensor]]],
+            type=List[Tuple[float, Union[np.array, coo_matrix, Tensor]]],
         ),
         forge.arg("duration", type=float),
         forge.arg("maximum_segment_duration", type=float),
         forge.arg("error_tolerance", type=float, default=1e-6),
         forge.arg("seed", type=Optional[int], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -132,7 +131,7 @@ class EstimatedKrylovSubspaceDimensionArnoldi(Node):
         error_tolerance = kwargs.get("error_tolerance")
 
         check_operator(hamiltonian_sample, "hamiltonian_sample")
-        check_lindblad_terms(lindblad_terms, hamiltonian_sample.shape[-1])
+        check_lindblad_terms(lindblad_terms, hamiltonian_sample, "hamiltonian_sample")
         check_duration(duration, "duration")
         check_duration(maximum_segment_duration, "maximum_segment_duration")
         check_argument(
@@ -141,12 +140,12 @@ class EstimatedKrylovSubspaceDimensionArnoldi(Node):
             {"error_tolerance": error_tolerance},
         )
 
-        return TensorNodeData(_operation, shape=())
+        return Tensor(_operation, shape=())
 
 
 class DensityMatrixEvolutionPwc(Node):
     r"""
-    Calculates the state evolution of an open system described by the GKS-Lindblad master
+    Calculates the state evolution of an open system described by the GKS–Lindblad master
     equation.
 
     The controls that you provide to this function have to be in piecewise constant
@@ -161,14 +160,15 @@ class DensityMatrixEvolutionPwc(Node):
         A 2D array of the shape ``[D, D]`` representing the initial density matrix of
         the system, :math:`\rho_{\rm s}`. You can also pass a batch of density matrices
         and the input data shape must be ``[B, D, D]`` where ``B`` is the batch dimension.
-    hamiltonian : Union[TensorPwc, SparsePwc]
+    hamiltonian : Union[Pwc, SparsePwc]
         A piecewise-constant function representing the effective system Hamiltonian,
-        :math:`H_{\rm s}(t)`, for the entire evolution duration. A sparse Hamiltonian gets
-        converted to a dense Hamiltonian.
+        :math:`H_{\rm s}(t)`, for the entire evolution duration. If you pass any Lindblad operator
+        as a dense array, the Hamiltonian will get converted to a (dense) Pwc.
     lindblad_terms : List[Tuple[float, Union[np.ndarray, scipy.sparse.coo_matrix]]]
         A list of pairs, :math:`(\gamma_j, L_j)`, representing the positive decay rate
         :math:`\gamma_j` and the Lindblad operator :math:`L_j` for each coupling
-        channel :math:`j`. Sparse operators get converted to dense operators.
+        channel :math:`j`. If you pass the Hamiltonian as a Pwc, the operators will get
+        converted to dense operators.
     sample_times : np.ndarray, optional
         A 1D array of length :math:`T` specifying the times :math:`\{t_i\}` at which this
         function calculates system states. Must be ordered and contain at least one element.
@@ -177,7 +177,6 @@ class DensityMatrixEvolutionPwc(Node):
         efficient calculation for large Hilbert spaces. A larger value leads to
         a better precision at the cost of longer computational time. If omitted or None
         it calculates the exact piecewise constant solution.
-
     name : str, optional
         The name of the node.
 
@@ -190,7 +189,7 @@ class DensityMatrixEvolutionPwc(Node):
 
     See Also
     --------
-    discretize_stf : Discretize an `Stf` into a `TensorPwc`.
+    discretize_stf : Discretize an `Stf` into a `Pwc`.
     estimated_krylov_subspace_dimension_arnoldi : Obtain a Krylov subspace dimension to use
         with this integrator.
     sparse_pwc_operator : Create `SparsePwc` operators.
@@ -199,7 +198,7 @@ class DensityMatrixEvolutionPwc(Node):
     Notes
     -----
     Under Markovian approximation, the dynamics of an open quantum system can be described by
-    the GKS-Lindblad master equation [1]_ [2]_
+    the GKS–Lindblad master equation [1]_ [2]_
 
     .. math::
         \frac{{\rm d}\rho_{\rm s}(t)}{{\rm d}t} = -i [H_{\rm s}(t), \rho_{\rm s}(t)]
@@ -212,12 +211,22 @@ class DensityMatrixEvolutionPwc(Node):
         {\mathcal D}[X]\rho := X \rho X^\dagger
             - \frac{1}{2}\left( X^\dagger X \rho + \rho X^\dagger X \right)
 
-    for any system operator :math:`X`. This function calculates the system state at the
-    sample times :math:`t_j` that you provide.
+    for any system operator :math:`X`.
+
+    This function calculates the system state at the sample times :math:`\{t_j\}` that you provide
+    by integrating the GKS–Lindblad equation via the Arnoldi algorithm. Note that increasing the
+    density of sample times does not affect the accuracy of the integration. However, increasing the
+    Krylov subspace dimension or subdividing the Hamiltonian into shorter piecewise-constant
+    segments can reduce the integration error, at the expense of longer computation times.
+
+    Moreover, this function uses sparse matrix multiplication when the Hamiltonian is passed as a
+    `SparsePwc` and the Lindblad operators as sparse matrices. This can lead to more efficient
+    calculations when they involve large operators that are relatively sparse (contain mostly
+    zeros). In this case, the initial density matrix is still a densely represented array or tensor.
 
     References
     ----------
-    .. [1] `V. Gorini, A. Kossakowski, E. C. G. Sudarshan,
+    .. [1] `V. Gorini, A. Kossakowski, and E. C. G. Sudarshan,
             Journal of Mathematical Physics 17, 821 (1976).
             <https://doi.org/10.1063/1.522979>`_
     .. [2] `G. Lindblad, Communications in Mathematical Physics 48, 119 (1976).
@@ -225,19 +234,19 @@ class DensityMatrixEvolutionPwc(Node):
     """
     name = "density_matrix_evolution_pwc"
     args = [
-        forge.arg("initial_density_matrix", type=Union[types.Tensor, np.ndarray]),
-        forge.arg("hamiltonian", type=Union[types.TensorPwc, types.SparsePwc]),
+        forge.arg("initial_density_matrix", type=Union[Tensor, np.ndarray]),
+        forge.arg("hamiltonian", type=Union[Pwc, SparsePwc]),
         forge.arg(
             "lindblad_terms", type=List[Tuple[float, Union[np.ndarray, coo_matrix]]]
         ),
         forge.arg("sample_times", type=Optional[np.ndarray], default=None),
         forge.arg(
             "krylov_subspace_dimension",
-            type=Union[int, types.Tensor, None],
+            type=Union[int, Tensor, None],
             default=None,
         ),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -247,15 +256,22 @@ class DensityMatrixEvolutionPwc(Node):
         lindblad_terms = kwargs.get("lindblad_terms")
 
         check_argument(
-            isinstance(hamiltonian, (TensorPwcNodeData, SparsePwcNodeData)),
-            "Hamiltonian must be a TensorPwc or a SparsePwc.",
+            isinstance(hamiltonian, (Pwc, SparsePwc)),
+            "Hamiltonian must be a Pwc or a SparsePwc.",
             {"hamiltonian": hamiltonian},
         )
+        check_argument(
+            isinstance(initial_density_matrix, (np.ndarray, Tensor)),
+            "Initial density matrix must be a NumPy array or a Tensor.",
+            {"initial_density_matrix": initial_density_matrix},
+        )
         if sample_times is not None:
             check_sample_times(sample_times, "sample_times")
         check_density_matrix_shape(initial_density_matrix, "initial_density_matrix")
-        check_oqs_hamiltonian(hamiltonian, initial_density_matrix.shape[-1])
-        check_lindblad_terms(lindblad_terms, initial_density_matrix.shape[-1])
+        check_oqs_hamiltonian(hamiltonian, initial_density_matrix)
+        check_lindblad_terms(
+            lindblad_terms, initial_density_matrix, "initial_density_matrix"
+        )
 
         initial_state_shape = initial_density_matrix.shape
         if sample_times is None:
@@ -266,4 +282,4 @@ class DensityMatrixEvolutionPwc(Node):
                 + (len(sample_times),)
                 + initial_state_shape[-2:]
             )
-        return TensorNodeData(_operation, shape=shape)
+        return Tensor(_operation, shape=shape)
