@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -13,6 +13,10 @@
 """Module for NodeRegistry."""
 import inspect
 from collections import defaultdict
+from typing import List
+
+from qctrlcommons.node import node_data
+from qctrlcommons.node.base import Node
 
 # pylint:disable=unused-wildcard-import, wildcard-import
 from .attribute import *
@@ -20,14 +24,25 @@ from .binary import *
 from .core import *
 from .infidelity import *
 from .ms import *
-from .optimization_variable import *
+from .optimization import *
 from .oqs import *
 from .pwc import *
 from .sparse import *
 from .stf import *
+from .stochastic import *
 from .tensorflow import *
 from .unary import *
 
+# Registry of all types created by operations.
+TYPE_REGISTRY = [
+    node_data.ConvolutionKernel,
+    node_data.Pwc,
+    node_data.SparsePwc,
+    node_data.Stf,
+    node_data.Target,
+    node_data.Tensor,
+]
+
 
 class NodeRegistry:
     """Register all the imported Node type class."""
@@ -45,7 +60,7 @@ class NodeRegistry:
             self._nodes[node.name] = node
 
     def get_node_cls(self, name: str) -> Node.__class__:
-        """Get the Node class by name from the pythonflow operation.
+        """Get the Node class by name from the operation.
 
         Parameters
         ----------
