@@ -1,17 +1,16 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
 """Module for sparse."""
-import warnings
 from typing import (
     List,
     Optional,
@@ -22,12 +21,11 @@ import forge
 import numpy as np
 from scipy.sparse import coo_matrix
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
 from qctrlcommons.node.node_data import (
-    SparsePwcNodeData,
-    TensorNodeData,
-    TensorPwcNodeData,
+    Pwc,
+    SparsePwc,
+    Tensor,
 )
 from qctrlcommons.node.utils import (
     validate_hamiltonian,
@@ -53,7 +51,7 @@ class SparsePwcOperator(Node):
 
     Parameters
     ----------
-    signal : TensorPwc
+    signal : Pwc
         The scalar-valued piecewise-constant function of time :math:`a(t)`.
     operator : numpy.ndarray or scipy.sparse.coo_matrix
         The sparse operator :math:`A` to be scaled over time. If you pass a NumPy array
@@ -66,31 +64,30 @@ class SparsePwcOperator(Node):
 
     See Also
     --------
-    constant_sparse_pwc_operator : Create constant `SparsePwc`s.
+    constant_sparse_pwc_operator : Create constant `SparsePwc`\s.
     density_matrix_evolution_pwc : Evolve a quantum state in an open system.
-    pwc_operator : Corresponding operation for `TensorPwc`s.
-    sparse_pwc_sum : Sum multiple `SparsePwc`s.
+    pwc_operator : Corresponding operation for `Pwc`\s.
+    sparse_pwc_sum : Sum multiple `SparsePwc`\s.
     state_evolution_pwc : Evolve a quantum state.
     """
 
     name = "sparse_pwc_operator"
     args = [
-        forge.arg("signal", type=types.TensorPwc),
+        forge.arg("signal", type=Pwc),
         forge.arg("operator", type=Union[np.ndarray, coo_matrix]),
     ]
     kwargs = {}  # SparsePwc doesn't accept `name` as an argument.
-    rtype = types.SparsePwc
+    rtype = SparsePwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         signal = kwargs.get("signal")
         operator = kwargs.get("operator")
-        signal_shape = signal.values_shape
         check_argument(
-            signal_shape == (),
+            signal.value_shape == (),
             "The signal must be scalar-valued.",
             {"signal": signal},
-            {"signal.values_shape": signal_shape},
+            {"signal.value_shape": signal.value_shape},
         )
         batch_shape = signal.batch_shape
         check_argument(
@@ -106,7 +103,7 @@ class SparsePwcOperator(Node):
             {"operator": operator},
             {"operator.shape": operator_shape},
         )
-        return SparsePwcNodeData(_operation, values_shape=operator_shape)
+        return SparsePwc(_operation, value_shape=operator_shape)
 
 
 class ConstantSparsePwcOperator(Node):
@@ -128,9 +125,9 @@ class ConstantSparsePwcOperator(Node):
 
     See Also
     --------
-    constant_pwc_operator : Corresponding operation for `TensorPwc`s.
+    constant_pwc_operator : Corresponding operation for `Pwc`\s.
     sparse_pwc_operator : Create `SparsePwc` operators.
-    sparse_pwc_sum : Sum multiple `SparsePwc`s.
+    sparse_pwc_sum : Sum multiple `SparsePwc`\s.
     """
 
     name = "constant_sparse_pwc_operator"
@@ -139,7 +136,7 @@ class ConstantSparsePwcOperator(Node):
         forge.arg("operator", type=Union[np.ndarray, coo_matrix]),
     ]
     kwargs = {}  # SparsePwc doesn't accept `name` as an argument.
-    rtype = types.SparsePwc
+    rtype = SparsePwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -153,7 +150,7 @@ class ConstantSparsePwcOperator(Node):
             {"operator": operator},
             {"operator.shape": operator_shape},
         )
-        return SparsePwcNodeData(_operation, values_shape=operator_shape)
+        return SparsePwc(_operation, value_shape=operator_shape)
 
 
 class SparsePwcSum(Node):
@@ -176,32 +173,32 @@ class SparsePwcSum(Node):
 
     See Also
     --------
-    constant_sparse_pwc_operator : Create constant `SparsePwc`s.
-    pwc_sum : Corresponding operation for `TensorPwc`s.
+    constant_sparse_pwc_operator : Create constant `SparsePwc`\s.
+    pwc_sum : Corresponding operation for `Pwc`\s.
     sparse_pwc_operator : Create `SparsePwc` operators.
     """
 
     name = "sparse_pwc_sum"
-    args = [forge.arg("terms", type=List[types.SparsePwc])]
+    args = [forge.arg("terms", type=List[SparsePwc])]
     kwargs = {}  # SparsePwcSum doesn't accept `name` as an argument.
-    rtype = types.SparsePwc
+    rtype = SparsePwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         terms = kwargs.get("terms")
-        shape_0 = terms[0].values_shape
+        shape_0 = terms[0].value_shape
         for i in range(1, len(terms)):
-            shape = terms[i].values_shape
+            shape = terms[i].value_shape
             check_argument(
                 shape == shape_0,
                 "All the terms must have the same shape.",
                 {"terms": terms},
                 {
-                    f"terms[{0}].values_shape": shape_0,
-                    f"terms[{i}].values_shape": shape,
+                    f"terms[{0}].value_shape": shape_0,
+                    f"terms[{i}].value_shape": shape,
                 },
             )
-        return SparsePwcNodeData(_operation, values_shape=shape_0)
+        return SparsePwc(_operation, value_shape=shape_0)
 
 
 class StateEvolutionPwc(Node):
@@ -213,7 +210,7 @@ class StateEvolutionPwc(Node):
     ----------
     initial_state : Union[Tensor, np.ndarray]
         The initial state as a Tensor or np.ndarray of shape ``[D]``.
-    hamiltonian : Union[TensorPwc, SparsePwc]
+    hamiltonian : Union[Pwc, SparsePwc]
         The control Hamiltonian. Uses sparse matrix multiplication if of type
         `SparsePwc`, which can be more efficient for large operators that are
         relatively sparse (contain mostly zeros).
@@ -243,21 +240,21 @@ class StateEvolutionPwc(Node):
     first try using `time_evolution_operators_pwc` to get the full time evolution
     operators (and evolve your state using those), and only switch to this method
     if that approach proves too slow or memory intensive. See the
-    `Simulate quantum dynamics user guide`_ for an example of calculating state
-    evolution with `time_evolution_operators_pwc`.
+    `How to simulate quantum dynamics for noiseless systems using graphs`_ user guide
+    for an example of calculating state evolution with `time_evolution_operators_pwc`.
 
-    .. _Simulate quantum dynamics user guide:
-        https://docs.q-ctrl.com/boulder-opal/user-guides/simulate-quantum-dynamics#Example-Generating-entangled-states-in-atomic-chains
+    .. _How to simulate quantum dynamics for noiseless systems using graphs:
+        https://docs.q-ctrl.com/boulder-opal/user-guides/how-to-simulate-quantum-dynamics-for-noiseless-systems-using-graphs
 
     See Also
     --------
     density_matrix_evolution_pwc : Corresponding operation for open systems.
-    discretize_stf : Discretize an `Stf` into a `TensorPwc`.
+    discretize_stf : Discretize an `Stf` into a `Pwc`.
     estimated_krylov_subspace_dimension_lanczos : Obtain a Krylov subspace dimension to use
         with this integrator.
     sparse_pwc_operator : Create `SparsePwc` operators.
     time_evolution_operators_pwc : Unitary time evolution operators for quantum systems with
-        `TensorPwc` Hamiltonians.
+        `Pwc` Hamiltonians.
 
     Notes
     -----
@@ -267,16 +264,26 @@ class StateEvolutionPwc(Node):
     much smaller that the full Hilbert space dimension. This allows for an efficient
     state propagation in high-dimensional systems compared to calculating the full
     unitary operator.
+
+    Moreover, this function uses sparse matrix multiplication when the Hamiltonian is passed as a
+    `SparsePwc`. This can lead to more efficient calculations when they involve large operators that
+    are relatively sparse (contain mostly zeros). In this case, the initial state is still a densely
+    represented array or tensor.
+
+    Note that increasing the density of `sample_times` does not affect the accuracy of the
+    integration. However, increasing the Krylov subspace dimension or subdividing the Hamiltonian
+    into shorter piecewise-constant segments can reduce the integration error, at the expense of
+    longer computation times.
     """
 
     name = "state_evolution_pwc"
     args = [
-        forge.arg("initial_state", type=Union[types.Tensor, np.ndarray]),
-        forge.arg("hamiltonian", type=Union[types.TensorPwc, types.SparsePwc]),
-        forge.arg("krylov_subspace_dimension", type=Union[int, types.Tensor]),
+        forge.arg("initial_state", type=Union[Tensor, np.ndarray]),
+        forge.arg("hamiltonian", type=Union[Pwc, SparsePwc]),
+        forge.arg("krylov_subspace_dimension", type=Union[int, Tensor]),
         forge.arg("sample_times", type=Optional[np.ndarray], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -287,76 +294,27 @@ class StateEvolutionPwc(Node):
         if sample_times is not None:
             check_sample_times(sample_times, "sample_times")
         check_argument(
-            isinstance(hamiltonian, (SparsePwcNodeData, TensorPwcNodeData)),
-            "The Hamiltonian must be a SparsePwc or a TensorPwc.",
+            isinstance(hamiltonian, (SparsePwc, Pwc)),
+            "The Hamiltonian must be a SparsePwc or a Pwc.",
             {"hamiltonian": hamiltonian},
         )
-        values_shape_state = validate_shape(initial_state, "initial_state")
-        values_shape_hamiltonian = hamiltonian.values_shape
+        value_shape_state = validate_shape(initial_state, "initial_state")
+        value_shape_hamiltonian = hamiltonian.value_shape
         validate_hamiltonian(hamiltonian, "hamiltonian")
         check_argument(
-            values_shape_state[0] == values_shape_hamiltonian[-1],
+            value_shape_state[0] == value_shape_hamiltonian[-1],
             "The Hilbert space dimensions of the state and the Hamiltonian must be equal.",
             {"initial_state": initial_state, "hamiltonian": hamiltonian},
             extras={
-                "initial_state.shape": values_shape_state,
-                "hamiltonian.values_shape": values_shape_hamiltonian,
+                "initial_state.shape": value_shape_state,
+                "hamiltonian.value_shape": value_shape_hamiltonian,
             },
         )
         if sample_times is None:
-            shape = values_shape_state
+            shape = value_shape_state
         else:
-            shape = (len(sample_times),) + values_shape_state
-        return TensorNodeData(_operation, shape=shape)
-
-
-class EstimatedKrylovSubspaceDimension(Node):
-    r"""
-    This node will be removed in the future.
-    Please use :py:func:`estimated_krylov_subspace_dimension_lanczos` instead.
-    """
-
-    name = "estimated_krylov_subspace_dimension"
-    args = [
-        forge.arg("spectral_range", type=Union[types.Tensor, float]),
-        forge.arg("duration", type=float),
-        forge.arg("maximum_segment_duration", type=float),
-        forge.arg("error_tolerance", type=float, default=1e-6),
-    ]
-    rtype = types.Tensor
-
-    @classmethod
-    def create_node_data(cls, _operation, **kwargs):
-        warnings.warn(
-            "The estimated_krylov_subspace_dimension node will be removed in the future. "
-            "Please use estimated_krylov_subspace_dimension_lanczos instead."
-        )
-        duration = kwargs.get("duration")
-        maximum_segment_duration = kwargs.get("maximum_segment_duration")
-        error_tolerance = kwargs.get("error_tolerance")
-        check_argument(
-            maximum_segment_duration > 0,
-            "The maximum segment duration must be positive.",
-            {"maximum_segment_duration": maximum_segment_duration},
-        )
-        check_argument(
-            duration > 0, "The duration must be positive.", {"duration": duration}
-        )
-        check_argument(
-            maximum_segment_duration <= duration,
-            "The maximum segment duration must be less than or equal to duration.",
-            {
-                "maximum_segment_duration": maximum_segment_duration,
-                "duration": duration,
-            },
-        )
-        check_argument(
-            error_tolerance > 0,
-            "The error tolerance must be positive.",
-            {"error_tolerance": error_tolerance},
-        )
-
-        return TensorNodeData(_operation, shape=())
+            shape = (len(sample_times),) + value_shape_state
+        return Tensor(_operation, shape=shape)
 
 
 class EstimatedKrylovSubspaceDimensionLanczos(Node):
@@ -421,12 +379,12 @@ class EstimatedKrylovSubspaceDimensionLanczos(Node):
 
     name = "estimated_krylov_subspace_dimension_lanczos"
     args = [
-        forge.arg("spectral_range", type=Union[types.Tensor, float]),
+        forge.arg("spectral_range", type=Union[Tensor, float]),
         forge.arg("duration", type=float),
         forge.arg("maximum_segment_duration", type=float),
         forge.arg("error_tolerance", type=float, default=1e-6),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -455,7 +413,7 @@ class EstimatedKrylovSubspaceDimensionLanczos(Node):
             {"error_tolerance": error_tolerance},
         )
 
-        return TensorNodeData(_operation, shape=())
+        return Tensor(_operation, shape=())
 
 
 class SpectralRange(Node):
@@ -521,11 +479,11 @@ class SpectralRange(Node):
 
     name = "spectral_range"
     args = [
-        forge.arg("operator", type=Union[types.Tensor, np.ndarray, coo_matrix]),
+        forge.arg("operator", type=Union[Tensor, np.ndarray, coo_matrix]),
         forge.arg("iteration_count", type=int, default=3000),
         forge.arg("seed", type=Optional[int], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -545,6 +503,6 @@ class SpectralRange(Node):
             "The number of iterations must be greater than zero.",
             {"iteration_count": iteration_count},
         )
-        if not isinstance(operator, TensorNodeData):
+        if not isinstance(operator, Tensor):
             check_argument_hermitian(operator, "operator")
-        return TensorNodeData(_operation, shape=())
+        return Tensor(_operation, shape=())
