@@ -1,18 +1,19 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
 # License for the specific language.
-"""Module for all the node related to stf."""
+"""
+Module for all the node related to stf.
+"""
 from typing import (
-    Callable,
     List,
     Optional,
     Union,
@@ -21,16 +22,21 @@ from typing import (
 import forge
 import numpy as np
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.deprecation import (
+    deprecated_node,
+    deprecated_parameter,
+)
 from qctrlcommons.node.node_data import (
-    StfNodeData,
-    TensorNodeData,
-    TensorPwcNodeData,
+    ConvolutionKernel,
+    Pwc,
+    Stf,
+    Tensor,
 )
 from qctrlcommons.node.utils import (
-    validate_batch_and_values_shapes,
+    validate_batch_and_value_shapes,
     validate_hamiltonian,
+    validate_inputs_real_fourier_signal,
     validate_shape,
 )
 from qctrlcommons.preconditions import (
@@ -38,13 +44,14 @@ from qctrlcommons.preconditions import (
     check_argument_integer,
     check_argument_iteratable,
     check_argument_numeric,
+    check_duration,
     check_operator,
     check_sample_times,
 )
 
 
 class StfOperator(Node):
-    """
+    r"""
     Creates a constant operator multiplied by a sampleable signal.
 
     Parameters
@@ -64,39 +71,39 @@ class StfOperator(Node):
     See Also
     --------
     constant_stf_operator : Create a constant `Stf` operator.
-    pwc_operator : Corresponding operation for `TensorPwc`s.
-    stf_sum : Sum multiple `Stf`s.
+    pwc_operator : Corresponding operation for `Pwc`\s.
+    stf_sum : Sum multiple `Stf`\s.
     """
 
     name = "stf_operator"
     args = [
-        forge.arg("signal", type=types.Stf),
-        forge.arg("operator", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("signal", type=Stf),
+        forge.arg("operator", type=Union[np.ndarray, Tensor]),
     ]
     kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = types.Stf
+    rtype = Stf
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         signal = kwargs.get("signal")
         operator = kwargs.get("operator")
         check_argument(
-            isinstance(signal, StfNodeData),
+            isinstance(signal, Stf),
             "The signal must be an Stf.",
             {"signal": signal},
         )
-        batch_shape, _ = validate_batch_and_values_shapes(signal, "signal")
-        values_shape = validate_shape(operator, "operator")
+        batch_shape, _ = validate_batch_and_value_shapes(signal, "signal")
+        value_shape = validate_shape(operator, "operator")
         check_operator(operator, "operator")
         check_argument(
-            len(values_shape) == 2,
+            len(value_shape) == 2,
             "The operator must be a matrix, not a batch.",
             {"operator": operator},
-            extras={"operator.shape": values_shape},
+            extras={"operator.shape": value_shape},
         )
-        return StfNodeData(
+        return Stf(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             batch_shape=batch_shape,
         )
 
@@ -119,24 +126,24 @@ class ConstantStfOperator(Node):
 
     See Also
     --------
-    constant_pwc_operator : Corresponding operation for `TensorPwc`s.
-    constant_stf: Create a batch of constant `Stf`s.
+    constant_pwc_operator : Corresponding operation for `Pwc`\s.
+    constant_stf: Create a batch of constant `Stf`\s.
     stf_operator : Create an `Stf` operator.
     """
 
     name = "constant_stf_operator"
-    args = [forge.arg("operator", type=Union[np.ndarray, types.Tensor])]
+    args = [forge.arg("operator", type=Union[np.ndarray, Tensor])]
     kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = types.Stf
+    rtype = Stf
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         operator = kwargs.get("operator")
         shape = validate_shape(operator, "operator")
         check_operator(operator, "operator")
-        return StfNodeData(
+        return Stf(
             _operation,
-            values_shape=shape[-2:],
+            value_shape=shape[-2:],
             batch_shape=shape[:-2],
         )
 
@@ -147,7 +154,7 @@ class ConstantStf(Node):
 
     Parameters
     ----------
-    constant : np.ndarray or Tensor
+    constant : number or np.ndarray or Tensor
         The constant value :math:`c` of the function.
         To create a batch of :math:`B_1 \times \ldots \times B_n` constant
         functions of shape :math:`D_1 \times \ldots \times D_m`, provide this `constant`
@@ -166,18 +173,18 @@ class ConstantStf(Node):
 
     See Also
     --------
+    constant_pwc : Corresponding operation for `Pwc`\s.
     constant_stf_operator : Create a constant sampleable function from operators.
     identity_stf : Create an `Stf` representing the identity function.
-    constant_pwc : Corresponding operation for `TensorPwc`s.
     """
 
     name = "constant_stf"
     args = [
-        forge.arg("constant", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("constant", type=Union[float, complex, np.ndarray, Tensor]),
         forge.arg("batch_dimension_count", type=int, default=0),
     ]
     kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = types.Stf
+    rtype = Stf
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -190,10 +197,10 @@ class ConstantStf(Node):
         shape = validate_shape(constant, "constant")
         check_argument(
             len(shape) >= batch_dimension_count,
-            "The number of batch dimensions must not be larger than the number of "
-            "dimensions of the input constant.",
+            "The number of batch dimensions must not be larger than the number "
+            "of dimensions of the input constant.",
             {"constant": constant, "batch_dimension_count": batch_dimension_count},
-            {"Number of value dimensions": len(constant.shape)},
+            {"Number of value dimensions": len(shape)},
         )
         check_argument(
             batch_dimension_count >= 0,
@@ -201,9 +208,9 @@ class ConstantStf(Node):
             {"batch_dimension_count": batch_dimension_count},
         )
 
-        return StfNodeData(
+        return Stf(
             operation=_operation,
-            values_shape=shape[batch_dimension_count:],
+            value_shape=shape[batch_dimension_count:],
             batch_shape=shape[:batch_dimension_count],
         )
 
@@ -225,33 +232,34 @@ class StfSum(Node):
 
     See Also
     --------
-    pwc_sum : Corresponding operation for `TensorPwc`s.
+    pwc_sum : Corresponding operation for `Pwc`\s.
     stf_operator : Create an `Stf` operator.
-    stf_sum : Sum multiple `Stf`s.
+    stf_operator_hermitian_part : Hermitian part of an `Stf` operator.
+    stf_sum : Sum multiple `Stf`\s.
     """
 
     name = "stf_sum"
-    args = [forge.arg("terms", type=List[types.Stf])]
+    args = [forge.arg("terms", type=List[Stf])]
     kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = types.Stf
+    rtype = Stf
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         terms = kwargs.get("terms")
         check_argument_iteratable(terms, "terms")
         check_argument(
-            all(isinstance(term, StfNodeData) for term in terms),
+            all(isinstance(term, Stf) for term in terms),
             "Each of the terms must be an Stf.",
             {"terms": terms},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             terms[0],
             "terms[0]",
         )
         check_argument(
             all(
                 (
-                    (values_shape == term.values_shape)
+                    (value_shape == term.value_shape)
                     and (batch_shape == term.batch_shape)
                 )
                 for term in terms[1:]
@@ -259,9 +267,50 @@ class StfSum(Node):
             "All the terms must have the same shape.",
             {"terms": terms},
         )
-        return StfNodeData(
+        return Stf(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
+            batch_shape=batch_shape,
+        )
+
+
+class StfOperatorHermitianPart(Node):
+    r"""
+    Creates the Hermitian part of an operator-valued sampleable function.
+
+    Parameters
+    ----------
+    operator : Stf
+        The operator-valued sampleable function :math:`A(t)`.
+
+    Returns
+    -------
+    Stf
+        The Hermitian part of the Stf operator :math:`\frac{1}{2}(A(t) + A^\dagger(t))`.
+
+    See Also
+    --------
+    pwc_operator_hermitian_part : Corresponding operation for `Pwc`\s.
+    stf_operator : Create an `Stf` operator.
+    """
+
+    name = "stf_operator_hermitian_part"
+    args = [forge.arg("operator", type=Stf)]
+    kwargs = {}  # Stfs don't accept name as an argument.
+    rtype = Stf
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        operator = kwargs.get("operator")
+        check_argument(
+            isinstance(operator, Stf),
+            "The operator must be an Stf.",
+            {"operator": operator},
+        )
+        batch_shape, value_shape = validate_batch_and_value_shapes(operator, "operator")
+        return Stf(
+            _operation,
+            value_shape=value_shape,
             batch_shape=batch_shape,
         )
 
@@ -285,10 +334,10 @@ class DiscretizeStf(Node):
         The duration :math:`\tau` over which discretization should be
         performed. The resulting piecewise-constant function has this
         duration.
-    segments_count : int
+    segment_count : int
         The number of segments :math:`N` in the resulting piecewise-constant
         function.
-    samples_per_segment : int, optional
+    sample_count_per_segment : int, optional
         The number of samples :math:`M` of the sampleable function to take when
         calculating the value of each segment in the discretization. Defaults
         to 1.
@@ -297,7 +346,7 @@ class DiscretizeStf(Node):
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The piecewise-constant function :math:`w(t)` obtained by discretizing
         the sampleable function (or batch of piecewise-constant functions, if
         you provided a batch of sampleable functions).
@@ -318,46 +367,46 @@ class DiscretizeStf(Node):
 
     See Also
     --------
-    convolve_pwc : Create an `Stf` by convolving a `TensorPwc` with a kernel.
+    convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
     identity_stf : Create an `Stf` representing the identity function.
     sample_stf : Sample an `Stf` at given times.
     """
 
     name = "discretize_stf"
     args = [
-        forge.arg("stf", type=types.Stf),
+        forge.arg("stf", type=Stf),
         forge.arg("duration", type=float),
-        forge.arg("segments_count", type=int),
-        forge.arg("samples_per_segment", type=int, default=1),
+        forge.arg("segment_count", type=int),
+        forge.arg("sample_count_per_segment", type=int, default=1),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         stf = kwargs.get("stf")
         duration = kwargs.get("duration")
-        segments_count = kwargs.get("segments_count")
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        segment_count = kwargs.get("segment_count")
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             stf,
             "stf",
         )
-        check_argument_integer(segments_count, "segments_count")
+        check_argument_integer(segment_count, "segment_count")
         check_argument(
-            segments_count > 0,
+            segment_count > 0,
             "The number of segments must be greater than zero.",
-            {"segments_count": segments_count},
+            {"segment_count": segment_count},
         )
-        durations = duration / segments_count * np.ones(segments_count)
-        samples_per_segment = kwargs.get("samples_per_segment")
-        check_argument_integer(samples_per_segment, "samples_per_segment")
+        durations = duration / segment_count * np.ones(segment_count)
+        sample_count_per_segment = kwargs.get("sample_count_per_segment")
+        check_argument_integer(sample_count_per_segment, "sample_count_per_segment")
         check_argument(
-            samples_per_segment > 0,
+            sample_count_per_segment > 0,
             "The number of samples per segment to take must be greater than zero.",
-            {"samples_per_segment": samples_per_segment},
+            {"sample_count_per_segment": sample_count_per_segment},
         )
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=durations,
             batch_shape=batch_shape,
         )
@@ -392,16 +441,16 @@ class TimeEvolutionOperatorsStf(Node):
 
     See Also
     --------
-    time_evolution_operators_pwc : Corresponding operation for `TensorPwc` Hamiltonians.
+    time_evolution_operators_pwc : Corresponding operation for `Pwc` Hamiltonians.
     """
 
     name = "time_evolution_operators_stf"
     args = [
-        forge.arg("hamiltonian", type=types.Stf),
+        forge.arg("hamiltonian", type=Stf),
         forge.arg("sample_times", type=np.ndarray),
         forge.arg("evolution_times", type=Optional[np.ndarray], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -409,11 +458,11 @@ class TimeEvolutionOperatorsStf(Node):
         sample_times = kwargs.get("sample_times")
         evolution_times = kwargs.get("evolution_times")
         check_argument(
-            isinstance(hamiltonian, StfNodeData),
+            isinstance(hamiltonian, Stf),
             "The Hamiltonian must be an Stf.",
             {"hamiltonian": hamiltonian},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             hamiltonian,
             "hamiltonian",
         )
@@ -434,23 +483,25 @@ class TimeEvolutionOperatorsStf(Node):
                 " times must be zero.",
                 {"sample_times": sample_times},
             )
-        shape = batch_shape + (time_count,) + values_shape
-        return TensorNodeData(_operation, shape=shape)
+        shape = batch_shape + (time_count,) + value_shape
+        return Tensor(_operation, shape=shape)
 
 
+@deprecated_parameter(  # deprecated 2021/05/24
+    deprecated_name="kernel_integral", new_name="kernel"
+)
 class ConvolvePwc(Node):
     r"""
     Creates the convolution of a piecewise-constant function with a kernel.
 
     Parameters
     ----------
-    pwc : TensorPwc
+    pwc : Pwc
         The piecewise-constant function :math:`\alpha(t)` to convolve. You
         can provide a batch of functions, in which case the convolution is
         applied to each element of the batch.
-    kernel_integral : Function
-        The node representing the function that computes the integral of the
-        kernel :math:`K(t)`.
+    kernel : ConvolutionKernel
+        The node representing the kernel :math:`K(t)`.
 
     Returns
     -------
@@ -473,39 +524,37 @@ class ConvolvePwc(Node):
 
     See Also
     --------
-    discretize_stf : Discretize an `Stf` into a `TensorPwc`.
-    gaussian_integral_function : Create a function that computes the integral of a normalized
-        Gaussian.
+    discretize_stf : Discretize an `Stf` into a `Pwc`.
+    gaussian_convolution_kernel : Create a convolution kernel representing a normalized Gaussian.
+    pwc : Create piecewise-constant functions.
     sample_stf : Sample an `Stf` at given times.
-    sinc_integral_function : Create a function that computes the integral of the sinc function.
-    tensor_pwc : Create piecewise-constant functions.
+    sinc_convolution_kernel : Create a convolution kernel representing the sinc function.
     """
 
     name = "convolve_pwc"
     args = [
-        forge.arg("pwc", type=types.TensorPwc),
-        forge.arg("kernel_integral", type=Callable),
+        forge.arg("pwc", type=Pwc),
+        forge.arg("kernel", type=ConvolutionKernel),
     ]
     kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = types.Stf
+    rtype = Stf
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         pwc = kwargs.get("pwc")
-        batch_shape, values_shape = validate_batch_and_values_shapes(pwc, "pwc")
-        return StfNodeData(
+        batch_shape, value_shape = validate_batch_and_value_shapes(pwc, "pwc")
+        return Stf(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             batch_shape=batch_shape,
         )
 
 
-class SincIntegralFunction(Node):
+class SincConvolutionKernel(Node):
     r"""
-    Creates a function that computes the integral of the sinc function.
+    Creates a convolution kernel representing the sinc function.
 
-    Use this function to create a filter kernel that eliminates frequencies
-    that are above a certain cut-off.
+    Use this kernel to eliminate frequencies above a certain cut-off.
 
     Parameters
     ----------
@@ -516,9 +565,8 @@ class SincIntegralFunction(Node):
 
     Returns
     -------
-    Function
-        A node representing a function that computes the integral of the sinc
-        function.
+    ConvolutionKernel
+        A node representing the sinc function to use in a convolution.
 
     Notes
     -----
@@ -532,24 +580,38 @@ class SincIntegralFunction(Node):
 
     The function on the right side of the equation is the sinc function.
     Its integral is the sine integral function (Si).
+
+    See Also
+    --------
+    convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
+    gaussian_convolution_kernel : Create a convolution kernel representing a normalized Gaussian.
     """
 
-    name = "sinc_integral_function"
+    name = "sinc_convolution_kernel"
     args = [
-        forge.arg("cut_off_frequency", type=Union[float, types.Tensor]),
+        forge.arg("cut_off_frequency", type=Union[float, Tensor]),
     ]
-    kwargs = {}  # Functions don't accept name as an argument.
-    rtype = types.Function
+    kwargs = {}  # ConvolutionKernels don't accept name as an argument.
+    rtype = ConvolutionKernel
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        return ConvolutionKernel(_operation)
 
 
-class GaussianIntegralFunction(Node):
+@deprecated_node("sinc_convolution_kernel")  # deprecated 2021/05/24
+class SincIntegralFunction(SincConvolutionKernel):
+    """Deprecated node."""
+
+    name = "sinc_integral_function"
+
+
+class GaussianConvolutionKernel(Node):
     r"""
-    Creates a function that computes the integral of a normalized Gaussian.
+    Creates a convolution kernel representing a normalized Gaussian.
 
-    Use this function to create a filter kernel that has a Gaussian shape. A
-    Gaussian kernel lets pass frequencies in the range roughly determined
-    by its width, and progressively suppresses components outside that
-    range.
+    Use this kernel to allow frequencies in the range roughly determined by
+    its width, and progressively suppress components outside that range.
 
     Parameters
     ----------
@@ -563,14 +625,12 @@ class GaussianIntegralFunction(Node):
 
     Returns
     -------
-    Function
-        A node representing a function that computes the integral of the
-        Gaussian function.
+    ConvolutionKernel
+        A node representing a Gaussian function to use in a convolution.
 
     Notes
     -----
-    The Gaussian that this function integrates is normalized in the time
-    domain:
+    The Gaussian that this node represents is normalized in the time domain:
 
     .. math::
         \frac{e^{-(t-\mu)^2/(2\sigma^2)}}{\sqrt{2\pi\sigma^2}}.
@@ -578,15 +638,31 @@ class GaussianIntegralFunction(Node):
     In the frequency domain, this Gaussian has standard deviation
     :math:`\omega_c= \sigma^{-1}`. The filter it represents therefore
     passes frequencies roughly in the range :math:`[-\omega_c, \omega_c]`.
+
+    See Also
+    --------
+    convolve_pwc : Create an `Stf` by convolving a `Pwc` with a kernel.
+    sinc_convolution_kernel : Create a convolution kernel representing the sinc function.
     """
 
-    name = "gaussian_integral_function"
+    name = "gaussian_convolution_kernel"
     args = [
-        forge.arg("std", type=Union[float, types.Tensor]),
-        forge.arg("offset", type=Optional[Union[float, types.Tensor]], default=0),
+        forge.arg("std", type=Union[float, Tensor]),
+        forge.arg("offset", type=Optional[Union[float, Tensor]], default=0),
     ]
-    kwargs = {}  # Functions don't accept name as an argument.
-    rtype = types.Function
+    kwargs = {}  # ConvolutionKernels don't accept name as an argument.
+    rtype = ConvolutionKernel
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        return ConvolutionKernel(_operation)
+
+
+@deprecated_node("gaussian_convolution_kernel")  # deprecated 2021/05/24
+class GaussianIntegralFunction(GaussianConvolutionKernel):
+    """Deprecated node."""
+
+    name = "gaussian_integral_function"
 
 
 class SampleStf(Node):
@@ -611,33 +687,32 @@ class SampleStf(Node):
     See Also
     --------
     constant_stf_operator : Create a constant `Stf` operator.
-    discretize_stf : Discretize an `Stf` into a `TensorPwc`.
+    discretize_stf : Discretize an `Stf` into a `Pwc`.
     identity_stf : Create an `Stf` representing the identity function.
+    sample_pwc : Sample a `Pwc` at given times.
     """
 
     name = "sample_stf"
     args = [
-        forge.arg("stf", type=types.Stf),
+        forge.arg("stf", type=Stf),
         forge.arg("sample_times", type=np.ndarray),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         stf = kwargs.get("stf")
-        check_argument(
-            isinstance(stf, StfNodeData), "The stf must be an Stf.", {"stf": stf}
-        )
-        batch_shape, values_shape = validate_batch_and_values_shapes(stf, "stf")
+        check_argument(isinstance(stf, Stf), "The stf must be an Stf.", {"stf": stf})
+        batch_shape, value_shape = validate_batch_and_value_shapes(stf, "stf")
         sample_times = kwargs.get("sample_times")
         check_sample_times(sample_times, "sample_times")
         time_count = len(sample_times)
-        shape = batch_shape + (time_count,) + values_shape
-        return TensorNodeData(_operation, shape=shape)
+        shape = batch_shape + (time_count,) + value_shape
+        return Tensor(_operation, shape=shape)
 
 
 class IdentityStf(Node):
-    """
+    r"""
     Returns an Stf representing the identity function, f(t) = t.
 
     Returns
@@ -647,20 +722,132 @@ class IdentityStf(Node):
 
     See Also
     --------
-    constant_stf: Create a batch of constant `Stf`s.
-    discretize_stf : Discretize an `Stf` into a `TensorPwc`.
+    constant_stf: Create a batch of constant `Stf`\s.
+    discretize_stf : Discretize an `Stf` into a `Pwc`.
     sample_stf : Sample an `Stf` at given times.
     """
 
     name = "identity_stf"
     args = []
     kwargs = {}  # Stfs don't accept name as an argument.
-    rtype = types.Stf
+    rtype = Stf
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        return StfNodeData(
+        return Stf(
             _operation,
-            values_shape=(),
+            value_shape=(),
+            batch_shape=(),
+        )
+
+
+class RealFourierStfSignal(Node):
+    r"""
+    Creates a real sampleable signal constructed from Fourier components.
+
+    Use this function to create a signal defined in terms of Fourier (sine/cosine)
+    basis signals that can be optimized by varying their coefficients and, optionally,
+    their frequencies.
+
+    Parameters
+    ----------
+    duration : float
+        The total duration :math:`\tau` of the signal.
+    initial_coefficient_lower_bound : float, optional
+        The lower bound :math:`c_\mathrm{min}` on the initial coefficient
+        values. Defaults to -1.
+    initial_coefficient_upper_bound : float, optional
+        The upper bound :math:`c_\mathrm{max}` on the initial coefficient
+        values. Defaults to 1.
+    fixed_frequencies : list[float], optional
+        The fixed non-zero frequencies :math:`\{f_m\}` to use for the Fourier
+        basis. Must be non-empty if provided. Must be specified in the inverse
+        units of `duration` (for example if `duration` is in seconds, these
+        values must be given in Hertz).
+    optimizable_frequency_count : int, optional
+        The number of non-zero frequencies :math:`M` to use, if the
+        frequencies can be optimized. Must be greater than zero if provided.
+    randomized_frequency_count : int, optional
+        The number of non-zero frequencies :math:`M` to use, if the
+        frequencies are to be randomized but fixed. Must be greater than zero
+        if provided.
+
+    Returns
+    -------
+    Stf(1D, real)
+        The optimizable, real-valued, sampleable signal built from the
+        appropriate Fourier components.
+
+    Warnings
+    --------
+    You must provide exactly one of `fixed_frequencies`, `optimizable_variable`,
+    or `randomized_frequency_count`.
+
+    See Also
+    --------
+    real_fourier_pwc_signal : Corresponding operation for `Pwc`.
+
+    Notes
+    -----
+    This function sets the basis signal frequencies :math:`\{f_m\}`
+    depending on the chosen mode:
+
+    * For fixed frequencies, you provide the frequencies directly.
+    * For optimizable frequencies, you provide the number of frequencies
+      :math:`M`, and this function creates :math:`M` unbounded optimizable
+      variables :math:`\{f_m\}`, with initial values in the ranges
+      :math:`\{[(m-1)/\tau, m/\tau]\}`.
+    * For randomized frequencies, you provide the number of frequencies
+      :math:`M`, and this function creates :math:`M` randomized constants
+      :math:`\{f_m\}` in the ranges :math:`\{[(m-1)/\tau, m/\tau]\}`.
+
+    After this function creates the :math:`M` frequencies :math:`\{f_m\}`, it
+    produces the signal
+
+    .. math::
+        \alpha^\prime(t) = v_0 +
+        \sum_{m=1}^M [ v_m \cos(2\pi t f_m) + w_m \sin(2\pi t f_m) ],
+
+    where :math:`\{v_m,w_m\}` are (unbounded) optimizable variables, with
+    initial values bounded by :math:`c_\mathrm{min}` and
+    :math:`c_\mathrm{max}`. This function produces the final signal :math:`\alpha(t)`.
+
+    You can use the signals created by this function for chopped random basis
+    (CRAB) optimization [1]_.
+
+    References
+    ----------
+    .. [1] `P. Doria, T. Calarco, and S. Montangero, Physical Review Letters 106, 190501 (2011).
+           <https://doi.org/10.1103/PhysRevLett.106.190501>`_
+    """
+
+    name = "real_fourier_stf_signal"
+    optimizable_variable = True
+    args = [
+        forge.arg("duration", type=float),
+        forge.arg("initial_coefficient_lower_bound", type=float, default=-1),
+        forge.arg("initial_coefficient_upper_bound", type=float, default=1),
+        forge.arg("fixed_frequencies", type=Optional[List[float]], default=None),
+        forge.arg("optimizable_frequency_count", type=Optional[int], default=None),
+        forge.arg("randomized_frequency_count", type=Optional[int], default=None),
+    ]
+    kwargs = {}  # Stfs don't accept name as an argument.
+    rtype = Stf
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        duration = kwargs.get("duration")
+        check_duration(duration, "duration")
+
+        fixed_frequencies = kwargs.get("fixed_frequencies")
+        optimizable_frequency_count = kwargs.get("optimizable_frequency_count")
+        randomized_frequency_count = kwargs.get("randomized_frequency_count")
+        validate_inputs_real_fourier_signal(
+            fixed_frequencies, optimizable_frequency_count, randomized_frequency_count
+        )
+
+        return Stf(
+            _operation,
+            value_shape=(),
             batch_shape=(),
         )
