@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -16,7 +16,11 @@ from typing import Union
 import numpy as np
 
 from qctrlcommons.exceptions import QctrlException
-from qctrlcommons.node import types
+from qctrlcommons.node.node_data import (
+    Pwc,
+    Stf,
+    Tensor,
+)
 from qctrlcommons.preconditions import (
     check_argument,
     check_argument_integer,
@@ -24,9 +28,7 @@ from qctrlcommons.preconditions import (
     check_argument_operator,
 )
 
-NumericOrFunction = Union[
-    float, complex, np.ndarray, types.Tensor, types.TensorPwc, types.Stf
-]
+NumericOrFunction = Union[float, complex, np.ndarray, Tensor, Pwc, Stf]
 
 
 def is_broadcastable(shape_a, shape_b) -> bool:
@@ -107,29 +109,29 @@ def validate_broadcasted_shape(x_shape, y_shape, x_name, y_name):
 
 def validate_function_output_shapes(
     x_batch_shape,
-    x_values_shape,
+    x_value_shape,
     y_batch_shape,
-    y_values_shape,
-    validate_values_shape,
+    y_value_shape,
+    validate_value_shape,
 ):
     """
-    Gets the output batch and values shape for two input shapes of Pwcs/Stfs.
+    Gets the output batch and value shape for two input shapes of Pwcs/Stfs.
     The names of the variables are assumed to be x and y when reporting errors.
 
     Parameters
     ----------
     x_batch_shape : tuple[int]
         The batch shape of the first object.
-    x_values_shape : tuple[int]
-        The values shape of the first object.
+    x_value_shape : tuple[int]
+        The value shape of the first object.
     y_batch_shape : tuple[int]
         The batch shape of the second object.
-    y_values_shape : tuple[int]
-        The values shape of the second object.
-    validate_values_shape : Callable[[tuple, tuple, str, str], tuple]
-        Function that takes the values shapes of two Tensors, TensorPwcs,
+    y_value_shape : tuple[int]
+        The value shape of the second object.
+    validate_value_shape : Callable[[tuple, tuple, str, str], tuple]
+        Function that takes the value shapes of two Tensors, Pwcs,
         or Stfs (as well as their names), and returns the expected values
-        shape of the output Tensor, TensorPwc, or Stf. The function
+        shape of the output Tensor, Pwc, or Stf. The function
         shouldn't assume that the shapes are compatible, and raise an
         exception if they aren't. The names provided should be used to
         generate the error message.
@@ -137,7 +139,7 @@ def validate_function_output_shapes(
     Returns
     -------
     tuple[int], tuple[int]
-        The batch and values shapes of the output Pwc/Stf.
+        The batch and value shapes of the output Pwc/Stf.
 
     Raises
     ------
@@ -147,22 +149,22 @@ def validate_function_output_shapes(
     batch_shape = validate_broadcasted_shape(
         x_batch_shape, y_batch_shape, "x (batch)", "y (batch)"
     )
-    values_shape = validate_values_shape(x_values_shape, y_values_shape, "x", "y")
+    value_shape = validate_value_shape(x_value_shape, y_value_shape, "x", "y")
 
-    return batch_shape, values_shape
+    return batch_shape, value_shape
 
 
 def validate_tensor_and_function_output_shapes(
     t_shape,
     f_batch_shape,
-    f_values_shape,
+    f_value_shape,
     t_name,
     f_name,
-    validate_values_shape,
+    validate_value_shape,
     tensor_first=True,
 ):
     """
-    Gets the output batch and values shape for an input tensor and an input Pwc/Stf.
+    Gets the output batch and value shape for an input tensor and an input Pwc/Stf.
 
     Parameters
     ----------
@@ -170,18 +172,18 @@ def validate_tensor_and_function_output_shapes(
         The shape of the tensor.
     f_batch_shape : tuple[int]
         The batch shape of the Pwc/Stf.
-    f_values_shape : tuple[int]
-        The values shape of the Pwc/Stf.
+    f_value_shape : tuple[int]
+        The value shape of the Pwc/Stf.
     t_name : str
         The name of the tensor variable, used for the error message in case the shapes aren't
         compatible.
     f_name : str
         The name of the function variable, used for the error message in case the shapes aren't
         compatible.
-    validate_values_shape : Callable[[tuple, tuple, str, str], tuple]
-        Function that takes the values shapes of two Tensors, TensorPwcs,
+    validate_value_shape : Callable[[tuple, tuple, str, str], tuple]
+        Function that takes the value shapes of two Tensors, Pwcs,
         or Stfs (as well as their names), and returns the expected values
-        shape of the output Tensor, TensorPwc, or Stf. The function
+        shape of the output Tensor, Pwc, or Stf. The function
         shouldn't assume that the shapes are compatible, and raise an
         exception if they aren't. The names provided should be used to
         generate the error message.
@@ -191,7 +193,7 @@ def validate_tensor_and_function_output_shapes(
     Returns
     -------
     tuple[int], tuple[int]
-        The batch and values shapes of the output Pwc/Stf.
+        The batch and value shapes of the output Pwc/Stf.
 
     Raises
     ------
@@ -199,11 +201,11 @@ def validate_tensor_and_function_output_shapes(
         if the two objects aren't compatible.
     """
     if tensor_first:
-        values_shape = validate_values_shape(t_shape, f_values_shape, t_name, f_name)
+        value_shape = validate_value_shape(t_shape, f_value_shape, t_name, f_name)
     else:
-        values_shape = validate_values_shape(f_values_shape, t_shape, f_name, t_name)
+        value_shape = validate_value_shape(f_value_shape, t_shape, f_name, t_name)
 
-    return f_batch_shape, values_shape
+    return f_batch_shape, value_shape
 
 
 def validate_shape(tensor_like, tensor_like_name):
@@ -212,7 +214,7 @@ def validate_shape(tensor_like, tensor_like_name):
 
     Parameters
     ----------
-    tensor_like : number or np.ndarray or scipy.sparse.coo_matrix or TensorNodeData
+    tensor_like : number or np.ndarray or scipy.sparse.coo_matrix or Tensor
         The object whose shape you want to obtain.
     tensor_like_name : str
         The name of the `tensor_like`, used for error message in case the
@@ -238,68 +240,66 @@ def validate_shape(tensor_like, tensor_like_name):
     raise QctrlException(f"The type of {tensor_like_name}={tensor_like} is not valid.")
 
 
-def validate_batch_and_values_shapes(tensor, tensor_name):
+def validate_batch_and_value_shapes(tensor, tensor_name):
     """
-    Returns the batch and values shapes of TensorPwc or Stf.
+    Returns the batch and value shapes of Pwc or Stf.
 
-    Paremeters
+    Parameters
     ----------
-    tensor : TensorPwcNodeData or StfNodeData
-        The NodeData for the TensorPwc or Stf whose batch and values shapes
+    tensor : Pwc or Stf
+        The NodeData for the Pwc or Stf whose batch and value shapes
         you want to obtain.
     tensor_name : str
-        The name of the TensorPwc or Stf, used for the error message in
-        case `tensor` doesn't have a values shape.
+        The name of the Pwc or Stf, used for the error message in
+        case `tensor` doesn't have a value shape.
 
     Returns
     -------
     tuple[tuple[int]]
         A tuple with a tuple that represents the batch shape, and a tuple
-        that represents the values shape, in this sequence.
+        that represents the value shape, in this sequence.
 
     Raises
     ------
     QctrlException
-        if the input is neither a scalar, a TensorPwc, nor an Stf
+        if the input is neither a scalar, a Pwc, nor an Stf
     """
 
-    if hasattr(tensor, "values_shape") and hasattr(tensor, "batch_shape"):
-        return tuple(tensor.batch_shape), tuple(tensor.values_shape)
+    if hasattr(tensor, "value_shape") and hasattr(tensor, "batch_shape"):
+        return tuple(tensor.batch_shape), tuple(tensor.value_shape)
 
-    if hasattr(tensor, "values_shape"):
-        return (), tuple(tensor.values_shape)
+    if hasattr(tensor, "value_shape"):
+        return (), tuple(tensor.value_shape)
 
-    raise QctrlException(
-        f"The type of {tensor_name}={tensor} must be TensorPwc or Stf."
-    )
+    raise QctrlException(f"The type of {tensor_name}={tensor} must be Pwc or Stf.")
 
 
 def validate_hamiltonian(hamiltonian, hamiltonian_name):
     """
-    Checks whether a TensorPwc, Stf or SparsePwc contains values that are Hamiltonians.
+    Checks whether a Pwc, Stf or SparsePwc contains values that are Hamiltonians.
 
     Hamiltonians are two-dimensional and square.
 
     Parameters
     ----------
-    hamiltonian : TensorPwc or Stf or SparsePwc
+    hamiltonian : Pwc or Stf or SparsePwc
         The Hamiltonian to be tested.
     hamiltonian_name : str
         The name of the Hamiltonian, used in the error message.
     """
-    values_shape = getattr(hamiltonian, "values_shape", ())
+    value_shape = getattr(hamiltonian, "value_shape", ())
 
     check_argument(
-        len(values_shape) == 2,
+        len(value_shape) == 2,
         "The shape of the Hamiltonian must have 2 dimensions.",
         {hamiltonian_name: hamiltonian},
-        extras={f"{hamiltonian_name}.values_shape": values_shape},
+        extras={f"{hamiltonian_name}.value_shape": value_shape},
     )
     check_argument(
-        values_shape[-1] == values_shape[-2],
+        value_shape[-1] == value_shape[-2],
         "The dimensions of the Hamiltonian must have equal sizes.",
         {hamiltonian_name: hamiltonian},
-        extras={f"{hamiltonian_name}.values_shape": values_shape},
+        extras={f"{hamiltonian_name}.value_shape": value_shape},
     )
 
 
@@ -350,7 +350,7 @@ def check_density_matrix_shape(density_matrix, name):
 
     Parameters
     ----------
-    density_matrix : Union[TensorNodeData, np.ndarray]
+    density_matrix : Union[Tensor, np.ndarray]
         A density matrix.
     name : str
         Name of the density matrix.
@@ -371,33 +371,37 @@ def check_density_matrix_shape(density_matrix, name):
     )
 
 
-def check_oqs_hamiltonian(hamiltonian, system_dimension):
+def check_oqs_hamiltonian(hamiltonian, initial_density_matrix):
     """
     Check whether open quantum system (OQS) Hamiltonian is valid.
 
     Parameters
     ----------
-    hamiltonian : Union[TensorPwc, SparsePwc]
+    hamiltonian : Union[Pwc, SparsePwc]
         Effective system Hamiltonian.
-    system_dimension : int
-        System Hilbert space dimension.
+    initial_density_matrix : Union[np.ndarray, Tensor]
+        Initial density matrix of the system.
     """
+    system_dimension = initial_density_matrix.shape[-1]
     check_argument_integer(system_dimension, "system dimension")
     check_argument(
-        hamiltonian.values_shape == (system_dimension, system_dimension),
-        "The dimension of the Hamiltonian compatible with the dimension "
-        "of the system Hilbert space.",
+        hamiltonian.value_shape == (system_dimension, system_dimension),
+        "The dimension of the Hamiltonian must be compatible with the dimension "
+        "of the initial density matrix.",
         {
             "hamiltonian": hamiltonian,
+            "initial_density_matrix": initial_density_matrix,
         },
         extras={
-            "hamiltonian dimension": hamiltonian.values_shape,
-            "system dimension": system_dimension,
+            "hamiltonian dimension": hamiltonian.value_shape,
+            "initial density matrix dimension": initial_density_matrix.shape,
         },
     )
 
 
-def check_lindblad_terms(lindblad_terms, system_dimension):
+def check_lindblad_terms(
+    lindblad_terms, hilbert_space_object, hilbert_space_object_name
+):
     """
     Check whether Lindblad terms are valid.
 
@@ -405,10 +409,15 @@ def check_lindblad_terms(lindblad_terms, system_dimension):
     ----------
     lindblad_terms : List[Tuple[float, Union[np.ndarray, coo_matrix]]]
         Effective system Hamiltonian.
-    system_dimension : int
-        System Hilbert space dimension.
+    hilbert_space_object : Union[np.ndarray, Tensor, scipy.sparse.spmatrix]
+        Object whose last element of its shape is the system's Hilbert space dimension.
+    hilbert_space_object_name : str
+        The name of the `hilbert_space_object`, used in an error message in case the
+        inputs dimensions don't match.
     """
-    check_argument_integer(system_dimension, "system dimension")
+
+    system_dimension = hilbert_space_object.shape[-1]
+
     for index, (rate, operator) in enumerate(lindblad_terms):
         check_argument(
             rate > 0,
@@ -421,10 +430,47 @@ def check_lindblad_terms(lindblad_terms, system_dimension):
         check_argument(
             operator.shape[0] == system_dimension,
             "The dimension of Lindblad operator must be compatible with the dimension "
-            "of the system Hilbert space.",
+            f"of {hilbert_space_object_name}.",
             {"lindblad_terms": lindblad_terms},
             extras={
-                "lindblad operator dimension": operator.shape,
-                "system dimension": system_dimension,
+                "Lindblad operator shape": operator.shape,
+                f"{hilbert_space_object_name} shape": hilbert_space_object.shape,
             },
         )
+
+
+def validate_inputs_real_fourier_signal(
+    fixed_frequencies, optimizable_frequency_count, randomized_frequency_count
+):
+    """
+    Check if the inputs of real_fourier_pwc/stf_signal function are valid.
+    """
+    check_argument(
+        (fixed_frequencies is not None)
+        + (optimizable_frequency_count is not None)
+        + (randomized_frequency_count is not None)
+        == 1,
+        "Exactly one of `fixed_frequencies`, `optimizable_frequency_count` and "
+        "`randomized_frequency_count` must be provided.",
+        {
+            "fixed_frequencies": fixed_frequencies,
+            "optimizable_frequency_count": optimizable_frequency_count,
+            "randomized_frequency_count": randomized_frequency_count,
+        },
+    )
+    if optimizable_frequency_count is not None:
+        check_argument_integer(
+            optimizable_frequency_count, "optimizable_frequency_count"
+        )
+        check_argument(
+            optimizable_frequency_count > 0,
+            "The number of optimizable frequencies (if provided) must be greater than zero.",
+            {"optimizable_frequency_count": optimizable_frequency_count},
+        )
+    if randomized_frequency_count is not None:
+        check_argument_integer(randomized_frequency_count, "randomized_frequency_count")
+        check_argument(
+            randomized_frequency_count > 0,
+            "The number of randomized frequencies (if provided) must be greater than zero.",
+            {"randomized_frequency_count": randomized_frequency_count},
+        )
