@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -233,6 +233,50 @@ def check_argument_integer(argument, argument_name):
     )
 
 
+def check_argument_positive_integer(argument, argument_name):
+    """
+    Raises QctrlArgumentsValueError if the argument is not a strictly positive integer.
+    """
+    check_argument(
+        np.isscalar(argument),
+        f"{argument_name} must be scalar.",
+        {argument_name: argument},
+    )
+    check_argument(
+        np.isreal(argument) and argument % 1 == 0 and argument > 0,
+        f"{argument_name} must be a positive integer.",
+        {argument_name: argument},
+    )
+
+
+def check_argument_real_vector(argument, argument_name):
+    """
+    Raises QctrlArgumentsValueError if the argument is not a 1-D array of real numbers.
+    """
+    check_argument_numeric(argument, argument_name)
+
+    # This makes sure that extracting shape doesn't throw an error if the
+    # user passes a list. Instead, a list will fail the next test, with a
+    # more meaningful message.
+    shape = getattr(argument, "shape", ())
+
+    check_argument(
+        len(shape) == 1,
+        f"{argument_name} must be a one-dimensional array.",
+        {argument_name: argument},
+    )
+    check_argument(
+        len(argument) > 0,
+        f"{argument_name} must be an array with at least one element.",
+        {argument_name: argument},
+    )
+    check_argument(
+        np.all(np.isreal(argument)),
+        f"{argument_name} must be real numbers.",
+        {argument_name: argument},
+    )
+
+
 def check_argument_numeric(argument, argument_name):
     """
     Raises QctrlArgumentsValueError if the argument is a NumPy array of
@@ -249,11 +293,46 @@ def check_argument_numeric(argument, argument_name):
             )
 
 
+def check_argument_non_negative_scalar(argument, argument_name):
+    """
+    Raises QctrlArgumentsValueError if the argument is not a non-negative scalar.
+
+    In other words, the argument should be a real scalar which is either greater than
+    or equal to zero.
+    """
+    check_argument(
+        np.isscalar(argument),
+        f"{argument_name} must be scalar.",
+        {argument_name: argument},
+    )
+    check_argument(
+        np.isreal(argument),
+        f"{argument_name} must be real.",
+        {argument_name: argument},
+    )
+    check_argument(
+        argument >= 0,
+        f"{argument_name} must not be negative.",
+        {argument_name: argument},
+    )
+
+
+def check_argument_positive_scalar(argument, argument_name):
+    """
+    Raises QctrlArgumentsValueError if the argument is not a strictly positive scalar
+    number.
+    """
+    check_argument_non_negative_scalar(argument, argument_name)
+    check_argument(
+        argument != 0, f"{argument_name} cannot be zero.", {argument_name: argument}
+    )
+
+
 def check_duration(duration, duration_name):
     """
     Checks that the duration is valid.
 
-    A valid duration is a positive real scalar.
+    A valid duration is a positive real or a zero scalar.
 
     Parameters
     ----------
@@ -262,21 +341,7 @@ def check_duration(duration, duration_name):
     duration_name : str
         The name of the parameter of the duration.
     """
-    check_argument(
-        np.isscalar(duration),
-        f"{duration_name} must be scalar.",
-        {duration_name: duration},
-    )
-    check_argument(
-        np.isreal(duration),
-        f"{duration_name} must be real.",
-        {duration_name: duration},
-    )
-    check_argument(
-        duration >= 0,
-        f"{duration_name} must not be negative.",
-        {duration_name: duration},
-    )
+    check_argument_non_negative_scalar(duration, duration_name)
 
 
 def check_sample_times(sample_times, sample_times_name):
@@ -293,28 +358,7 @@ def check_sample_times(sample_times, sample_times_name):
     sample_times_name : str
         The name of the array.
     """
-    check_argument_numeric(sample_times, sample_times_name)
-
-    # This makes sure that extracting shape doesn't throw an error if the
-    # user passes a list. Instead, a list will fail the next test, with a
-    # more meaningful message.
-    shape = getattr(sample_times, "shape", ())
-
-    check_argument(
-        len(shape) == 1,
-        f"{sample_times_name} must be a one-dimensional array.",
-        {sample_times_name: sample_times},
-    )
-    check_argument(
-        len(sample_times) > 0,
-        f"{sample_times_name} must be an array with at least one element.",
-        {sample_times_name: sample_times},
-    )
-    check_argument(
-        np.all(np.isreal(sample_times)),
-        f"{sample_times_name} must be real numbers.",
-        {sample_times_name: sample_times},
-    )
+    check_argument_real_vector(sample_times, sample_times_name)
     check_argument(
         # note that this is also true for sample_time only holding one element or empty.
         np.all(np.diff(sample_times) > 0),
@@ -331,7 +375,7 @@ def check_operator(operator, operator_name):
 
     Parameters
     ----------
-    operator : np.ndarray or TensorNodeData
+    operator : np.ndarray or Tensor
         The operator to be tested for validity.
     operator_name : str
         The name of the operator, used for the error messages.
@@ -339,7 +383,7 @@ def check_operator(operator, operator_name):
     check_argument_numeric(operator, operator_name)
 
     # Obtains shape in a way that doesn't throw an obscure error message if
-    # the user passes a list or a TensorPwc by mistake. Instead, an error is
+    # the user passes a list or a Pwc by mistake. Instead, an error is
     # thrown in the next check_argument.
     shape = getattr(operator, "shape", ())
 
