@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -31,6 +31,7 @@ from .data_types import (
     NumpyComplexNumber,
     NumpyScalar,
     OperationDataType,
+    QuantumObject,
     SliceDataType,
     SparseMatrix,
 )
@@ -51,6 +52,7 @@ class DataTypeMixin:  # pylint:disable=too-few-public-methods
         SliceDataType(),
         SparseMatrix(),
         FloatConstant(),
+        QuantumObject(),
     ]
 
 
@@ -99,7 +101,7 @@ class DataTypeEncoder(json.JSONEncoder, DataTypeMixin):
     JavaScript constants use `qctrlcommons.serializers.encode`.
     """
 
-    def default(self, obj):  # pylint:disable=arguments-differ, method-hidden
+    def default(self, obj):  # pylint:disable=arguments-renamed, method-hidden
         # check if any data type can be encoded
         for data_type in self.data_types:
             if data_type.can_encode(obj):
